/*
 *  @version 	        @(#)ListLoc.java	Tutorial Solution 1.1.7
 *
 * Copyright (c) 1997-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */



import suntest.javastar.lib.*;
import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.MouseEvent;
/*
 *
 * Non Component Locator for JList in JFC Swing
*/

public class TheListLoc implements JSNonComponentLocator {

  public JSNCLData findObject(Component c, AWTEvent e){
// ensure that this is a list and a mouse event
	if(!(c instanceof JList) || !(e instanceof MouseEvent)){
		return null;
	}	
// get the point from the mouse event
      	Point mousePoint = ((MouseEvent)e).getPoint();
// cast the component to a JList
      	JList theList = (JList)c;
// obtain the index based on the location
    int theIndex = theList.locationToIndex(mousePoint);
// make sure the index exists
    if(theIndex != -1){
// use a common method to tranlate an index entry in a list to NCLData
      return getListData(theList,theIndex);
    }
    return null;
  }

  public JSNCLData getNamedObjectData(Component c, String wname){
// make sure this is a list
    if (!(c instanceof JList)){
	return null;
	}
// cast it to a JList
      JList theList = (JList)c;
// create a parser for this string
      Parser P = new Parser(wname);
// make sure the string is valid
      if (!P.Valid){
	throw new BadRegularExpressionError(wname + " is not a valid format");

      }
// get the index and the confirming name from the parser 

    int theIndex = P.Index;
    String theConfirmingName = P.Value;
// get the model for the list	
    ListModel theListModel = theList.getModel();
// confirm that the index is within the list
    int numberInList = theListModel.getSize();

    if(theIndex<=-1){
	return null;
    }
    if(theIndex>=numberInList){
	return null;
    }
    if(theConfirmingName == null){
	return getListData(theList, theIndex);
	}
//assert that the index is within the model, and that there is a confirming name to check for
    String name = Parser.cleanToString(theListModel.getElementAt(theIndex));
    if(theConfirmingName.equals(findName(theList, theIndex))){
// use the common method to translate a list and an index to that list to NCLData
	return getListData(theList,theIndex);
	} else {	   
    		return null;
	}	
  }

  public static JSNCLData getListData(JList aList, int anIndex){
	// get the model for the list
    ListModel aListModel = aList.getModel();
	// make sure the index is within the model size
    if(anIndex >=aListModel.getSize()) return null;
	// get the location of the index point
    Point theNCPoint = aList.indexToLocation(anIndex);
	// get the element for the object
	Object theNCObject = getObject(aList,anIndex);

    
	// get a clean string (no special characters) for this object
    String theObjectName = findName(aList, anIndex);
	// prepend the index and create the name
    String theNCName = anIndex + ":" + theObjectName;
    return new JSNCLData(theNCName, theNCPoint, theNCObject);
  }
static Object getObject(JList aList,int anIndex) {
	ListModel aListModel = aList.getModel();
	Object theNCObject = aListModel.getElementAt(anIndex);
	if (theNCObject instanceof Integer) {
      		ListCellRenderer renderer = aList.getCellRenderer();
      		if (renderer != null) {
			Component comp = renderer.getListCellRendererComponent
				(aList, theNCObject,
				 anIndex, aList.isSelectedIndex(anIndex),
				 aList.hasFocus());
			if (comp != null) {
	  			theNCObject = comp;
			}
      		}
	}
	return theNCObject;
}
static String findName(JList L, int idx) {
    ListModel M = L.getModel();
    Object o = M.getElementAt(idx);
    String name = null;
    if (o instanceof ImageIcon) {
      name = ((ImageIcon)o).getDescription();
    } else if (o instanceof Integer) {
      ListCellRenderer renderer = L.getCellRenderer();
      if (renderer != null) {
	Component comp = renderer.getListCellRendererComponent(L, o, idx, L.isSelectedIndex(idx), L.hasFocus());
	if (comp != null) {
	  o = comp;
	  if (comp.getName() != null && comp.getName().length() > 0) {
	    name = comp.getName();
	  } else {
	    if (comp instanceof JLabel)
	      name = ((JLabel)comp).getText();
	    else if (comp instanceof Label)
	      name = ((Label)comp).getText();
	  }
	}
      }
    }
    return (name != null ? name : Parser.cleanToString(o));
  }
}
